/* rbcrack.c - a Ruby interface to CrackLib
 * 
 * $Id: rbcrack.c,v 1.7 2002/09/28 20:07:05 ianmacd Exp $
 *
 * Version : 0.2.1
 * Author  : Ian Macdonald <ian@caliban.org>
 *
 * Copyright (C) 2002 Ian Macdonald
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software Foundation,
 *   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <ruby.h>

#ifdef HAVE_PACKER_H
#include <packer.h>
#else
#include <crack.h>
#endif

#include "rbcrack.h"

#define CRACK_VERSION "0.2.1"


VALUE eCrack_WeakPassword;

/* Crack.initialize */
static VALUE passwd_init(VALUE self, VALUE passwd)
{

  /* call superclass's initialize method */
  rb_call_super(1, &passwd);

  return passwd;
}

/* Crack.check */
static VALUE passwd_check(VALUE self, VALUE args)
{
  VALUE dict;
  char *objection;

  /* pop the one and only argument we may have been passed */
  dict = rb_ary_pop(args);

  if (dict == Qnil)
    /* no argument passed, so use default location from rbcrack.h */
    dict = rb_str_new2(CRACK_DICT);

  /* perform check on password */
  objection = FascistCheck(STR2CSTR(self), STR2CSTR(dict));

  /* return true on success; raise an exception otherwise */
  if (objection) {
    rb_raise(eCrack_WeakPassword, "%s", objection);
  } else {
    return Qtrue;
  }

}

/* initialize this class */
void Init_crack()
{
  VALUE cCrack;

  /* define the Crack class */
  cCrack = rb_define_class("Crack", rb_cString);

  /* define the Crack::WeakPassword exception */
  eCrack_WeakPassword = rb_define_class_under(cCrack, "WeakPassword",
		                              rb_eRuntimeError);

  /* define a version constant */
  rb_define_const(cCrack, "VERSION", rb_str_new2(CRACK_VERSION));

  /* define the Crack.initialize method */
  rb_define_method(cCrack, "initialize", passwd_init, 1);

  /* enable the same method in the superclass */
  rb_enable_super(cCrack, "initialize");

  /* define the Crack.check method */
  rb_define_method(cCrack, "check", passwd_check, -2);

  return;
}
